@@ -14,27 +14,29 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-from operator import attrgetter
+from __future__ import annotations
 
-from flask import current_app
+from operator import attrgetter
 
 from airflow import DAG
 from airflow.api_connexion import security
 from airflow.api_connexion.exceptions import BadRequest, NotFound
 from airflow.api_connexion.schemas.task_schema import TaskCollection, task_collection_schema, task_schema
+from airflow.api_connexion.types import APIResponse
 from airflow.exceptions import TaskNotFound
 from airflow.security import permissions
+from airflow.utils.airflow_flask_app import get_airflow_app
 
 
 @security.requires_access(
     [
         (permissions.ACTION_CAN_READ, permissions.RESOURCE_DAG),
         (permissions.ACTION_CAN_READ, permissions.RESOURCE_TASK_INSTANCE),
-    ]
+    ],
 )
-def get_task(dag_id, task_id):
+def get_task(*, dag_id: str, task_id: str) -> APIResponse:
     """Get simplified representation of a task."""
-    dag: DAG = current_app.dag_bag.get_dag(dag_id)
+    dag: DAG = get_airflow_app().dag_bag.get_dag(dag_id)
     if not dag:
         raise NotFound("DAG not found")
 
@@ -49,17 +51,17 @@ def get_task(dag_id, task_id):
     [
         (permissions.ACTION_CAN_READ, permissions.RESOURCE_DAG),
         (permissions.ACTION_CAN_READ, permissions.RESOURCE_TASK_INSTANCE),
-    ]
+    ],
 )
-def get_tasks(dag_id, order_by='task_id'):
-    """Get tasks for DAG"""
-    dag: DAG = current_app.dag_bag.get_dag(dag_id)
+def get_tasks(*, dag_id: str, order_by: str = "task_id") -> APIResponse:
+    """Get tasks for DAG."""
+    dag: DAG = get_airflow_app().dag_bag.get_dag(dag_id)
     if not dag:
         raise NotFound("DAG not found")
     tasks = dag.tasks
 
     try:
-        tasks = sorted(tasks, key=attrgetter(order_by.lstrip('-')), reverse=(order_by[0:1] == '-'))
+        tasks = sorted(tasks, key=attrgetter(order_by.lstrip("-")), reverse=(order_by[0:1] == "-"))
     except AttributeError as err:
         raise BadRequest(detail=str(err))
     task_collection = TaskCollection(tasks=tasks, total_entries=len(tasks))
