@@ -14,14 +14,16 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-"""Config sub-commands"""
+"""Config sub-commands."""
+from __future__ import annotations
+
 import locale
 import logging
 import os
 import platform
 import subprocess
 import sys
-from typing import List, Optional
+from enum import Enum
 from urllib.parse import urlsplit, urlunsplit
 
 import httpx
@@ -42,19 +44,19 @@ class Anonymizer(Protocol):
     """Anonymizer protocol."""
 
     def process_path(self, value) -> str:
-        """Remove pii from paths"""
+        """Remove pii from paths."""
 
     def process_username(self, value) -> str:
-        """Remove pii from username"""
+        """Remove pii from username."""
 
     def process_url(self, value) -> str:
-        """Remove pii from URL"""
+        """Remove pii from URL."""
 
 
 class NullAnonymizer(Anonymizer):
     """Do nothing."""
 
-    def _identity(self, value):
+    def _identity(self, value) -> str:
         return value
 
     process_path = process_username = process_url = _identity
@@ -70,19 +72,19 @@ class PiiAnonymizer(Anonymizer):
         username = getuser()
         self._path_replacements = {home_path: "${HOME}", username: "${USER}"}
 
-    def process_path(self, value):
+    def process_path(self, value) -> str:
         if not value:
             return value
         for src, target in self._path_replacements.items():
             value = value.replace(src, target)
         return value
 
-    def process_username(self, value):
+    def process_username(self, value) -> str:
         if not value:
             return value
         return value[0] + "..." + value[-1]
 
-    def process_url(self, value):
+    def process_url(self, value) -> str:
         if not value:
             return value
 
@@ -123,17 +125,18 @@ class PiiAnonymizer(Anonymizer):
         return urlunsplit((url_parts.scheme, netloc, url_parts.path, url_parts.query, url_parts.fragment))
 
 
-class OperatingSystem:
-    """Operating system"""
+class OperatingSystem(Enum):
+    """Operating system."""
 
     WINDOWS = "Windows"
     LINUX = "Linux"
     MACOSX = "Mac OS"
     CYGWIN = "Cygwin"
+    UNKNOWN = "Unknown"
 
     @staticmethod
-    def get_current() -> Optional[str]:
-        """Get current operating system"""
+    def get_current() -> OperatingSystem:
+        """Get current operating system."""
         if os.name == "nt":
             return OperatingSystem.WINDOWS
         elif "linux" in sys.platform:
@@ -142,24 +145,26 @@ class OperatingSystem:
             return OperatingSystem.MACOSX
         elif "cygwin" in sys.platform:
             return OperatingSystem.CYGWIN
-        return None
+        return OperatingSystem.UNKNOWN
 
 
-class Architecture:
-    """Compute architecture"""
+class Architecture(Enum):
+    """Compute architecture."""
 
     X86_64 = "x86_64"
     X86 = "x86"
     PPC = "ppc"
     ARM = "arm"
+    UNKNOWN = "unknown"
 
     @staticmethod
-    def get_current():
-        """Get architecture"""
-        return _MACHINE_TO_ARCHITECTURE.get(platform.machine().lower())
+    def get_current() -> Architecture:
+        """Get architecture."""
+        current_architecture = _MACHINE_TO_ARCHITECTURE.get(platform.machine().lower())
+        return current_architecture if current_architecture else Architecture.UNKNOWN
 
 
-_MACHINE_TO_ARCHITECTURE = {
+_MACHINE_TO_ARCHITECTURE: dict[str, Architecture] = {
     "amd64": Architecture.X86_64,
     "x86_64": Architecture.X86_64,
     "i686-64": Architecture.X86_64,
@@ -175,17 +180,18 @@ _MACHINE_TO_ARCHITECTURE = {
     "arm64": Architecture.ARM,
     "armv7": Architecture.ARM,
     "armv7l": Architecture.ARM,
+    "aarch64": Architecture.ARM,
 }
 
 
 class AirflowInfo:
-    """Renders information about Airflow instance"""
+    """Renders information about Airflow instance."""
 
     def __init__(self, anonymizer):
         self.anonymizer = anonymizer
 
     @staticmethod
-    def _get_version(cmd: List[str], grep: Optional[bytes] = None):
+    def _get_version(cmd: list[str], grep: bytes | None = None):
         """Return tools version."""
         try:
             with subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT) as proc:
@@ -209,10 +215,10 @@ class AirflowInfo:
             if module is None or module == str.__class__.__module__:
                 return o.__class__.__name__  # Avoid reporting __builtin__
             else:
-                return module + '.' + o.__class__.__name__
+                return module + "." + o.__class__.__name__
 
         try:
-            handler_names = [get_fullname(handler) for handler in logging.getLogger('airflow.task').handlers]
+            handler_names = [get_fullname(handler) for handler in logging.getLogger("airflow.task").handlers]
             return ", ".join(handler_names)
         except Exception:
             return "NOT AVAILABLE"
@@ -221,7 +227,7 @@ class AirflowInfo:
     def _airflow_info(self):
         executor = configuration.conf.get("core", "executor")
         sql_alchemy_conn = self.anonymizer.process_url(
-            configuration.conf.get("core", "SQL_ALCHEMY_CONN", fallback="NOT AVAILABLE")
+            configuration.conf.get("database", "SQL_ALCHEMY_CONN", fallback="NOT AVAILABLE")
         )
         dags_folder = self.anonymizer.process_path(
             configuration.conf.get("core", "dags_folder", fallback="NOT AVAILABLE")
@@ -257,8 +263,8 @@ class AirflowInfo:
         python_version = sys.version.replace("\n", " ")
 
         return [
-            ("OS", operating_system or "NOT AVAILABLE"),
-            ("architecture", arch or "NOT AVAILABLE"),
+            ("OS", operating_system.value),
+            ("architecture", arch.value),
             ("uname", str(uname)),
             ("locale", str(_locale)),
             ("python_version", python_version),
@@ -304,10 +310,10 @@ class AirflowInfo:
 
     @property
     def _providers_info(self):
-        return [(p.provider_info['package-name'], p.version) for p in ProvidersManager().providers.values()]
+        return [(p.data["package-name"], p.version) for p in ProvidersManager().providers.values()]
 
-    def show(self, output: str, console: Optional[AirflowConsole] = None) -> None:
-        """Shows information about Airflow instance"""
+    def show(self, output: str, console: AirflowConsole | None = None) -> None:
+        """Shows information about Airflow instance."""
         all_info = {
             "Apache Airflow": self._airflow_info,
             "System info": self._system_info,
@@ -329,7 +335,7 @@ class AirflowInfo:
             )
 
     def render_text(self, output: str) -> str:
-        """Exports the info to string"""
+        """Exports the info to string."""
         console = AirflowConsole(record=True)
         with console.capture():
             self.show(output=output, console=console)
@@ -337,7 +343,7 @@ class AirflowInfo:
 
 
 class FileIoException(Exception):
-    """Raises when error happens in FileIo.io integration"""
+    """Raises when error happens in FileIo.io integration."""
 
 
 @tenacity.retry(
@@ -348,7 +354,7 @@ class FileIoException(Exception):
     after=tenacity.after_log(log, logging.DEBUG),
 )
 def _upload_text_to_fileio(content):
-    """Upload text file to File.io service and return lnk"""
+    """Upload text file to File.io service and return link."""
     resp = httpx.post("https://file.io", content=content)
     if resp.status_code not in [200, 201]:
         print(resp.json())
