@@ -20,11 +20,12 @@ API and outputs a kubernetes.client.models.V1Pod.
 The advantage being that the full Kubernetes API
 is supported and no serialization need be written.
 """
+from __future__ import annotations
+
 import copy
 import hashlib
 import re
 import uuid
-from typing import Dict, List, Optional, Union
 
 from kubernetes.client import models as k8s
 
@@ -36,15 +37,15 @@ MAX_LABEL_LEN = 63
 class PodDefaults:
     """Static defaults for Pods"""
 
-    XCOM_MOUNT_PATH = '/airflow/xcom'
-    SIDECAR_CONTAINER_NAME = 'airflow-xcom-sidecar'
+    XCOM_MOUNT_PATH = "/airflow/xcom"
+    SIDECAR_CONTAINER_NAME = "airflow-xcom-sidecar"
     XCOM_CMD = 'trap "exit 0" INT; while true; do sleep 30; done;'
-    VOLUME_MOUNT = k8s.V1VolumeMount(name='xcom', mount_path=XCOM_MOUNT_PATH)
-    VOLUME = k8s.V1Volume(name='xcom', empty_dir=k8s.V1EmptyDirVolumeSource())
+    VOLUME_MOUNT = k8s.V1VolumeMount(name="xcom", mount_path=XCOM_MOUNT_PATH)
+    VOLUME = k8s.V1Volume(name="xcom", empty_dir=k8s.V1EmptyDirVolumeSource())
     SIDECAR_CONTAINER = k8s.V1Container(
         name=SIDECAR_CONTAINER_NAME,
-        command=['sh', '-c', XCOM_CMD],
-        image='alpine',
+        command=["sh", "-c", XCOM_CMD],
+        image="alpine",
         volume_mounts=[VOLUME_MOUNT],
         resources=k8s.V1ResourceRequirements(
             requests={
@@ -82,101 +83,73 @@ class PodGenerator:
     the first container in the list of containers.
 
     :param image: The docker image
-    :type image: Optional[str]
     :param name: name in the metadata section (not the container name)
-    :type name: Optional[str]
     :param namespace: pod namespace
-    :type namespace: Optional[str]
     :param volume_mounts: list of kubernetes volumes mounts
-    :type volume_mounts: Optional[List[Union[k8s.V1VolumeMount, dict]]]
     :param envs: A dict containing the environment variables
-    :type envs: Optional[Dict[str, str]]
     :param cmds: The command to be run on the first container
-    :type cmds: Optional[List[str]]
     :param args: The arguments to be run on the pod
-    :type args: Optional[List[str]]
     :param labels: labels for the pod metadata
-    :type labels: Optional[Dict[str, str]]
     :param node_selectors: node selectors for the pod
-    :type node_selectors: Optional[Dict[str, str]]
     :param ports: list of ports. Applies to the first container.
-    :type ports: Optional[List[Union[k8s.V1ContainerPort, dict]]]
     :param volumes: Volumes to be attached to the first container
-    :type volumes: Optional[List[Union[k8s.V1Volume, dict]]]
     :param image_pull_policy: Specify a policy to cache or always pull an image
-    :type image_pull_policy: str
     :param restart_policy: The restart policy of the pod
-    :type restart_policy: str
     :param image_pull_secrets: Any image pull secrets to be given to the pod.
         If more than one secret is required, provide a comma separated list:
         secret_a,secret_b
-    :type image_pull_secrets: str
     :param init_containers: A list of init containers
-    :type init_containers: Optional[List[k8s.V1Container]]
     :param service_account_name: Identity for processes that run in a Pod
-    :type service_account_name: Optional[str]
     :param resources: Resource requirements for the first containers
-    :type resources: Optional[Union[k8s.V1ResourceRequirements, dict]]
     :param annotations: annotations for the pod
-    :type annotations: Optional[Dict[str, str]]
     :param affinity: A dict containing a group of affinity scheduling rules
-    :type affinity: Optional[dict]
     :param hostnetwork: If True enable host networking on the pod
-    :type hostnetwork: bool
     :param tolerations: A list of kubernetes tolerations
-    :type tolerations: Optional[list]
     :param security_context: A dict containing the security context for the pod
-    :type security_context: Optional[Union[k8s.V1PodSecurityContext, dict]]
     :param configmaps: Any configmap refs to envfrom.
         If more than one configmap is required, provide a comma separated list
         configmap_a,configmap_b
-    :type configmaps: List[str]
     :param dnspolicy: Specify a dnspolicy for the pod
-    :type dnspolicy: Optional[str]
     :param schedulername: Specify a schedulername for the pod
-    :type schedulername: Optional[str]
     :param pod: The fully specified pod. Mutually exclusive with `path_or_string`
-    :type pod: Optional[kubernetes.client.models.V1Pod]
     :param extract_xcom: Whether to bring up a container for xcom
-    :type extract_xcom: bool
     :param priority_class_name: priority class name for the launched Pod
-    :type priority_class_name: str
     """
 
     def __init__(
         self,
-        image: Optional[str] = None,
-        name: Optional[str] = None,
-        namespace: Optional[str] = None,
-        volume_mounts: Optional[List[Union[k8s.V1VolumeMount, dict]]] = None,
-        envs: Optional[Dict[str, str]] = None,
-        cmds: Optional[List[str]] = None,
-        args: Optional[List[str]] = None,
-        labels: Optional[Dict[str, str]] = None,
-        node_selectors: Optional[Dict[str, str]] = None,
-        ports: Optional[List[Union[k8s.V1ContainerPort, dict]]] = None,
-        volumes: Optional[List[Union[k8s.V1Volume, dict]]] = None,
-        image_pull_policy: Optional[str] = None,
-        restart_policy: Optional[str] = None,
-        image_pull_secrets: Optional[str] = None,
-        init_containers: Optional[List[k8s.V1Container]] = None,
-        service_account_name: Optional[str] = None,
-        resources: Optional[Union[k8s.V1ResourceRequirements, dict]] = None,
-        annotations: Optional[Dict[str, str]] = None,
-        affinity: Optional[dict] = None,
+        image: str | None = None,
+        name: str | None = None,
+        namespace: str | None = None,
+        volume_mounts: list[k8s.V1VolumeMount | dict] | None = None,
+        envs: dict[str, str] | None = None,
+        cmds: list[str] | None = None,
+        args: list[str] | None = None,
+        labels: dict[str, str] | None = None,
+        node_selectors: dict[str, str] | None = None,
+        ports: list[k8s.V1ContainerPort | dict] | None = None,
+        volumes: list[k8s.V1Volume | dict] | None = None,
+        image_pull_policy: str | None = None,
+        restart_policy: str | None = None,
+        image_pull_secrets: str | None = None,
+        init_containers: list[k8s.V1Container] | None = None,
+        service_account_name: str | None = None,
+        resources: k8s.V1ResourceRequirements | dict | None = None,
+        annotations: dict[str, str] | None = None,
+        affinity: dict | None = None,
         hostnetwork: bool = False,
-        tolerations: Optional[list] = None,
-        security_context: Optional[Union[k8s.V1PodSecurityContext, dict]] = None,
-        configmaps: Optional[List[str]] = None,
-        dnspolicy: Optional[str] = None,
-        schedulername: Optional[str] = None,
+        tolerations: list | None = None,
+        security_context: k8s.V1PodSecurityContext | dict | None = None,
+        configmaps: list[str] | None = None,
+        dnspolicy: str | None = None,
+        schedulername: str | None = None,
         extract_xcom: bool = False,
-        priority_class_name: Optional[str] = None,
+        priority_class_name: str | None = None,
     ):
 
         self.pod = k8s.V1Pod()
-        self.pod.api_version = 'v1'
-        self.pod.kind = 'Pod'
+        self.pod.api_version = "v1"
+        self.pod.kind = "Pod"
 
         # Pod Metadata
         self.metadata = k8s.V1ObjectMeta()
@@ -186,7 +159,7 @@ class PodGenerator:
         self.metadata.annotations = annotations
 
         # Pod Container
-        self.container = k8s.V1Container(name='base')
+        self.container = k8s.V1Container(name="base")
         self.container.image = image
         self.container.env = []
 
@@ -206,7 +179,8 @@ class PodGenerator:
 
         self.container.command = cmds or []
         self.container.args = args or []
-        self.container.image_pull_policy = image_pull_policy
+        if image_pull_policy:
+            self.container.image_pull_policy = image_pull_policy
         self.container.ports = ports or []
         self.container.resources = resources
         self.container.volume_mounts = volume_mounts or []
@@ -215,7 +189,8 @@ class PodGenerator:
         self.spec = k8s.V1PodSpec(containers=[])
         self.spec.security_context = security_context
         self.spec.tolerations = tolerations
-        self.spec.dns_policy = dnspolicy
+        if dnspolicy:
+            self.spec.dns_policy = dnspolicy
         self.spec.scheduler_name = schedulername
         self.spec.host_network = hostnetwork
         self.spec.affinity = affinity
@@ -223,13 +198,14 @@ class PodGenerator:
         self.spec.init_containers = init_containers
         self.spec.volumes = volumes or []
         self.spec.node_selector = node_selectors
-        self.spec.restart_policy = restart_policy
+        if restart_policy:
+            self.spec.restart_policy = restart_policy
         self.spec.priority_class_name = priority_class_name
 
         self.spec.image_pull_secrets = []
 
         if image_pull_secrets:
-            for image_pull_secret in image_pull_secrets.split(','):
+            for image_pull_secret in image_pull_secrets.split(","):
                 self.spec.image_pull_secrets.append(k8s.V1LocalObjectReference(name=image_pull_secret))
 
         # Attach sidecar
@@ -265,7 +241,7 @@ class PodGenerator:
         return pod_cp
 
     @staticmethod
-    def from_obj(obj) -> Optional[k8s.V1Pod]:
+    def from_obj(obj) -> k8s.V1Pod | None:
         """Converts to pod from obj"""
         if obj is None:
             return None
@@ -275,8 +251,8 @@ class PodGenerator:
 
         if not isinstance(obj, dict):
             raise TypeError(
-                'Cannot convert a non-dictionary or non-PodGenerator '
-                'object into a KubernetesExecutorConfig'
+                "Cannot convert a non-dictionary or non-PodGenerator "
+                "object into a KubernetesExecutorConfig"
             )
 
         # We do not want to extract constant here from ExecutorLoader because it is just
@@ -286,25 +262,25 @@ class PodGenerator:
         if not namespaced:
             return None
 
-        resources = namespaced.get('resources')
+        resources = namespaced.get("resources")
 
         if resources is None:
             requests = {
-                'cpu': namespaced.get('request_cpu'),
-                'memory': namespaced.get('request_memory'),
-                'ephemeral-storage': namespaced.get('ephemeral-storage'),
+                "cpu": namespaced.get("request_cpu"),
+                "memory": namespaced.get("request_memory"),
+                "ephemeral-storage": namespaced.get("ephemeral-storage"),
             }
             limits = {
-                'cpu': namespaced.get('limit_cpu'),
-                'memory': namespaced.get('limit_memory'),
-                'ephemeral-storage': namespaced.get('ephemeral-storage'),
+                "cpu": namespaced.get("limit_cpu"),
+                "memory": namespaced.get("limit_memory"),
+                "ephemeral-storage": namespaced.get("ephemeral-storage"),
             }
             all_resources = list(requests.values()) + list(limits.values())
             if all(r is None for r in all_resources):
                 resources = None
             else:
                 resources = k8s.V1ResourceRequirements(requests=requests, limits=limits)
-        namespaced['resources'] = resources
+        namespaced["resources"] = resources
         return PodGenerator(**namespaced).gen_pod()
 
     @staticmethod
