@@ -15,20 +15,22 @@
 # specific language governing permissions and limitations
 # under the License.
 """DAG Cycle tester"""
-from collections import defaultdict, deque
-from typing import TYPE_CHECKING
+from __future__ import annotations
 
-from airflow.exceptions import AirflowDagCycleException
+from collections import defaultdict, deque
+from typing import TYPE_CHECKING, Deque
+
+from airflow.exceptions import AirflowDagCycleException, RemovedInAirflow3Warning
 
 if TYPE_CHECKING:
-    from airflow.models import DAG
+    from airflow.models.dag import DAG
 
 CYCLE_NEW = 0
 CYCLE_IN_PROGRESS = 1
 CYCLE_DONE = 2
 
 
-def test_cycle(dag: "DAG") -> None:
+def test_cycle(dag: DAG) -> None:
     """
     A wrapper function of `check_cycle` for backward compatibility purpose.
     New code should use `check_cycle` instead since this function name `test_cycle` starts with 'test_' and
@@ -38,27 +40,27 @@ def test_cycle(dag: "DAG") -> None:
 
     warn(
         "Deprecated, please use `check_cycle` at the same module instead.",
-        DeprecationWarning,
+        RemovedInAirflow3Warning,
         stacklevel=2,
     )
     return check_cycle(dag)
 
 
-def check_cycle(dag: "DAG") -> None:
+def check_cycle(dag: DAG) -> None:
     """Check to see if there are any cycles in the DAG.
 
     :raises AirflowDagCycleException: If cycle is found in the DAG.
     """
     # default of int is 0 which corresponds to CYCLE_NEW
-    visited = defaultdict(int)
-    path_stack = deque()
+    visited: dict[str, int] = defaultdict(int)
+    path_stack: Deque[str] = deque()
     task_dict = dag.task_dict
 
     def _check_adjacent_tasks(task_id, current_task):
         """Returns first untraversed child task, else None if all tasks traversed."""
         for adjacent_task in current_task.get_direct_relative_ids():
             if visited[adjacent_task] == CYCLE_IN_PROGRESS:
-                msg = f"Cycle detected in DAG. Faulty task: {task_id}"
+                msg = f"Cycle detected in DAG: {dag.dag_id}. Faulty task: {task_id}"
                 raise AirflowDagCycleException(msg)
             elif visited[adjacent_task] == CYCLE_NEW:
                 return adjacent_task
